-- This script generates associations between a general foreign property table
-- as generated by loadForeignProperties and WikiWord, using the value of 
-- a single specific property.
--
-- Parameters:
--  * wikiword_prefix: the global table prefix. Provided automatically
--  * foreign_properties_table: the table containing the foreign entities/properties
--  * foreign_authority_name: name of the foreign authority (scope if the foreign proeprty name)
--  * foreign_property_name: name of the property in the foreign property table.
--  * concept_property_name: name of the property in WikiWord's property table.

SELECT foreign_authority, foreign_id, 
	concept, concept_name, 
	F.property as foreign_property, P.property as concept_property,
	F.value as value,
	count(*) as concept_property_freq
FROM /* wikiword_prefix *//* foreign_properties_table */ as F
JOIN /* wikiword_prefix */property as P
ON F.value = P.value
AND F.foreign_authority = "/* foreign_authority_name */"
AND F.property = "/* foreign_property_name */"
AND P.property = "/* concept_property_name */"
GROUP BY foreign_authority, foreign_id, concept, F.value;